/*
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is MPEG4IP.
 * 
 * The Initial Developer of the Original Code is Cisco Systems Inc.
 * Portions created by Cisco Systems Inc. are
 * Copyright (C) Cisco Systems Inc. 2001.  All Rights Reserved.
 * 
 * Contributor(s): 
 *		Dave Mackie		dmackie@cisco.com
 */
/*
#include "mp4.h"

main(int argc, char** argv)
{
	MP4FileHandle mp4File = MP4Read(argv[1], MP4_DETAILS_ERROR);

	if (!mp4File) {
		exit(1);
	}

	MP4Dump(mp4File, stdout, 0);

	MP4Close(mp4File);

	exit(0);
}

*/
/* mp4tags -- tool to set iTunes-compatible metadata tags
 * 
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * Contributed to MPEG4IP 
 * by Christopher League <league@contrapunctus.net>
 */

#include "mp4.h"
//#include "mpeg4ip_getopt.h"

/* One-letter options -- if you want to rearrange these, change them
   here, immediately below in OPT_STRING, and in the help text. */
#define OPT_HELP    'h'
#define OPT_VERSION 'v'
#define OPT_ALBUM   'A'
#define OPT_ARTIST  'a'
#define OPT_TEMPO   'b'
#define OPT_COMMENT 'c'
#define OPT_DISK    'd'
#define OPT_DISKS   'D'
#define OPT_GENRE   'g'
#define OPT_SONG    's'
#define OPT_TRACK   't'
#define OPT_TRACKS  'T'
#define OPT_WRITER  'w'    /* _composer_ */
#define OPT_YEAR    'y'

#define OPT_STRING  "hvA:a:b:c:d:D:g:s:t:T:w:y:"

static const char* help_text =
"OPTION... FILE...\n"
"Adds or modifies iTunes-compatible tags on MP4 files.\n"
"\n"
"  -h, -help         Display this help text and exit\n"
"  -v, -version      Display version information and exit\n"
"  -A, -album   STR  Set the album title\n"
"  -a, -artist  STR  Set the artist information\n"
"  -b, -tempo   NUM  Set the tempo (beats per minute)\n"
"  -c, -comment STR  Set a general comment\n"
"  -d, -disk    NUM  Set the disk number\n"
"  -D, -disks   NUM  Set the number of disks\n"
"  -g, -genre   STR  Set the genre name\n"
"  -s, -song    STR  Set the song title\n"
"  -t, -track   NUM  Set the track number\n"
"  -T, -tracks  NUM  Set the number of tracks\n"
"  -w, -writer  STR  Set the composer information\n"
"  -y, -year    NUM  Set the year\n"
"\n";


int 
main(int argc, char** argv)
{
  /* Sparse arrays of tag data: some space is wasted, but it's more
     convenient to say tags[OPT_SONG] than to enumerate all the
     metadata types (again) as a struct. */
  char *tags[UCHAR_MAX] = { 0 };
  int nums[UCHAR_MAX] = { 0 };

  /* Any modifications requested? */
  int mods = 0;

    MP4FileHandle h = MP4Modify(argv[1], MP4_DETAILS_ALL);
    if (h == MP4_INVALID_FILE_HANDLE) {
      fprintf(stderr, "Could not open '%s'... aborting\n", argv[1]);
      return 5;
    }

    /* Track/disk numbers need to be set all at once, but we'd like to
       allow users to just specify -T 12 to indicate that all existing
       track numbers are out of 12.  This means we need to look up the
       current info if it is not being set. */
    uint16_t n0, m0, n1, m1;
    MP4GetMetadataTrack(h, &n0, &m0);
	char buff[512];
	char* album = buff;
	MP4GetMetadataAlbum(h, &album);

    if (tags[OPT_TRACK] || tags[OPT_TRACKS]) {
      MP4GetMetadataTrack(h, &n0, &m0);
      n1 = tags[OPT_TRACK]? nums[OPT_TRACK] : n0;
      m1 = tags[OPT_TRACKS]? nums[OPT_TRACKS] : m0;
      MP4SetMetadataTrack(h, n1, m1);
    }
    if (tags[OPT_DISK] || tags[OPT_DISKS]) {
      MP4GetMetadataDisk(h, &n0, &m0);
      n1 = tags[OPT_DISK]? nums[OPT_DISK] : n0;
      m1 = tags[OPT_DISKS]? nums[OPT_DISKS] : m0;
      MP4SetMetadataDisk(h, n1, m1);
    }
    
    /* Set the other relevant attributes */
	/*
    for (int i = 0;  i < UCHAR_MAX;  i++) {
      if (tags[i]) {
        switch(i) {
        case OPT_ALBUM:   MP4SetMetadataAlbum(h, tags[i]); break;
        case OPT_ARTIST:  MP4SetMetadataArtist(h, tags[i]); break;
        case OPT_COMMENT: MP4SetMetadataComment(h, tags[i]); break;
        case OPT_GENRE:   MP4SetMetadataGenre(h, tags[i]); break;
        case OPT_SONG:    MP4SetMetadataName(h, tags[i]); break;
        case OPT_WRITER:  MP4SetMetadataWriter(h, tags[i]); break;
        case OPT_YEAR:    MP4SetMetadataYear(h, tags[i]); break;
        case OPT_TEMPO:   MP4SetMetadataTempo(h, nums[i]); break;
        }
      }
    }
	*/
	u_int32_t index = 0;
	typedef struct {
		//char nameBuff[1024];
		char* name;
		//u_int8_t valueBuff[1024+1];
		char* value;
		u_int32_t size;
	} MetaData;
	MetaData meta[20];
	const char* name;
	u_int8_t* value;
	u_int32_t size;
	while (MP4GetMetadataByIndex(h,index, &name, &value, &size) == true) {
		meta[index].name = (char*)malloc(strlen(name)+1);
		strcpy(meta[index].name, name);
		meta[index].value = (char*)malloc(size + 1);
		memcpy(meta[index].value, value, size);
		meta[index].value[size] = '\0';
		meta[index].size = size;
		index++;
	}
	if (MP4GetMetadataCoverArt(h, &value, &size) == true) {
		meta[index].name = (char*)malloc(strlen("covr")+1);
		strcpy(meta[index].name, "covr");
		meta[index].value = (char*)malloc(size + 1);
		memcpy(meta[index].value, value, size);
		meta[index].value[size] = '\0';
		meta[index].size = size;
		index++;
	}
	MP4MetadataDelete(h);
	while (index > 0) {
		index--;
		if (!stricmp(meta[index].name, "trkn") || !stricmp(meta[index].name, "tracknumber"))
		{
//			unsigned __int16 trkn, tot;
//			int t1 = 0, t2 = 0;
//			sscanf(meta[index].value, "%d/%d", &t1, &t2);
//			trkn = t1, tot = t2;
//			if (!trkn;
//			MP4SetMetadataTrack(h, trkn, tot);
		}
		else if (!stricmp(meta[index].name, "disc") || !stricmp(meta[index].name, "disknumber"))
		{
//			unsigned __int16 disk, tot;
//			int t1 = 0, t2 = 0;
//			sscanf(meta[index].value, "%d/%d", &t1, &t2);
//			disk = t1, tot = t2;
//			if (!disk;
//			MP4SetMetadataDisk(h, disk, tot);
		}
		else if (!stricmp(meta[index].name, "cpil"))
		{
			unsigned __int8 cpil = atoi(meta[index].value);
			if (cpil != 0)
				MP4SetMetadataCompilation(h, cpil);
		}
		else if (!stricmp(meta[index].name, "tmpo"))
		{
			unsigned __int16 tempo = atoi(meta[index].value);
			if (tempo != 0)
				MP4SetMetadataTempo(h, tempo);
		}
		else if (!stricmp(meta[index].name, "ART"))
		{
			MP4SetMetadataArtist(h, meta[index].value);
		}
		else if (!stricmp(meta[index].name, "wrt"))
		{
			MP4SetMetadataWriter(h, meta[index].value);
		}
		else if (!stricmp(meta[index].name, "nam"))
		{
			MP4SetMetadataName(h, meta[index].value);
		}
		else if (!stricmp(meta[index].name, "alb"))
		{
			MP4SetMetadataAlbum(h, meta[index].value);
		}
		else if (!stricmp(meta[index].name, "day") || !stricmp(meta[index].name, "year"))
		{
			MP4SetMetadataYear(h, meta[index].value);
		}
		else if (!stricmp(meta[index].name, "cmt"))
		{
			MP4SetMetadataComment(h, meta[index].value);
		}
		else if (!stricmp(meta[index].name, "gen"))
		{
			MP4SetMetadataGenre(h, meta[index].value);
		}
		else if (!stricmp(meta[index].name, "too"))
		{
			MP4SetMetadataTool(h, meta[index].value);
		}
		else if (!stricmp(meta[index].name, "covr"))
		{
			MP4SetMetadataCoverArt(h, (u_int8_t*)meta[index].value, meta[index].size);
		}
		else
		{
			MP4SetMetadataFreeForm(h, meta[index].name, (u_int8_t*)meta[index].value, meta[index].size);
		}
	}
//	MP4SetMetadataAlbum(h, "aaaaaaaa");
    MP4Close(h);
   MP4Optimize(argv[1], NULL, MP4_DETAILS_ALL);

  return 0;
}
